@@ -7,4 +7,4 @@ use Lingua::NL::FactoidExtractor;
 my $inputfile = "alpino.xml";
 my $verbose = 1; #boolean
 my $factoids = extract($inputfile,$verbose);
-print $factoids;
+print "$factoids\n";
